@@ -49,8 +49,12 @@
 
 /* XXX fix race conditions. */
 
-#include "internal.h"
+#include "info.h"
 
+/* Get prototypes. */
+#include "file.h"
+
+/* Get low-level file utilities. */
 #include "fileutil.h"
 
 struct Shisa_file
@@ -62,11 +66,11 @@ struct Shisa_file
 typedef struct Shisa_file Shisa_file;
 
 enum
-  {
-    READ_ONLY_OPTION = 0,
-    ALLOW_CREATE_OPTION = 1,
-    THE_END
-  };
+{
+  READ_ONLY_OPTION = 0,
+  ALLOW_CREATE_OPTION = 1,
+  THE_END
+};
 
 static char *const _shisa_file_opts[] = {
   /* [READ_ONLY_OPTION] =        */ "read-only",
@@ -75,9 +79,7 @@ static char *const _shisa_file_opts[] = {
 };
 
 int
-shisa_file_cfg (Shisa *dbh,
-		Shisa_file *info,
-		const char *option)
+shisa_file_cfg (Shisa * dbh, Shisa_file * info, const char *option)
 {
   char *opt = option ? xstrdup (option) : NULL;
   char *p = opt;
@@ -108,11 +110,11 @@ shisa_file_cfg (Shisa *dbh,
   return SHISA_OK;
 }
 
+/* Initialize file backend, i.e., parse options and check if file root
+   exists and allocate backend handle. */
 int
-shisa_file_init (Shisa *dbh,
-		 const char *location,
-		 const char *options,
-		 void **state)
+shisa_file_init (Shisa * dbh,
+		 const char *location, const char *options, void **state)
 {
   Shisa_file *info;
   int rc;
@@ -131,11 +133,25 @@ shisa_file_init (Shisa *dbh,
   return SHISA_OK;
 }
 
+/* Destroy backend handle. */
+void
+shisa_file_done (Shisa * dbh, void *state)
+{
+  Shisa_file *info = state;
+
+  if (info)
+    {
+      if (info->path)
+	free (info->path);
+      free (info);
+    }
+}
+
+/* Return a list of all realm names in backend, as zero-terminated
+   UTF-8 strings.  The caller must deallocate the strings. */
 int
-shisa_file_enumerate_realms (Shisa *dbh,
-			     void *state,
-			     char ***realms,
-			     size_t *nrealms)
+shisa_file_enumerate_realms (Shisa * dbh,
+			     void *state, char ***realms, size_t * nrealms)
 {
   Shisa_file *info = state;
 
@@ -145,12 +161,14 @@ shisa_file_enumerate_realms (Shisa *dbh,
   return SHISA_OK;
 }
 
+/* Return a list of all principals in realm in backend, as
+   zero-terminated UTF-8 strings.  The caller must deallocate the
+   strings. */
 int
-shisa_file_enumerate_principals (Shisa *dbh,
+shisa_file_enumerate_principals (Shisa * dbh,
 				 void *state,
 				 const char *realm,
-				 char ***principals,
-				 size_t *nprincipals)
+				 char ***principals, size_t * nprincipals)
 {
   Shisa_file *info = state;
 
@@ -163,18 +181,22 @@ shisa_file_enumerate_principals (Shisa *dbh,
   return SHISA_OK;
 }
 
+/* Return information about specified PRINCIPAL@REALM.  Can also be
+   used check existence of principal entry, with a NULL PH. */
 int
 shisa_file_principal_find (Shisa * dbh,
 			   void *state,
 			   const char *realm,
-			   const char *principal,
-			   Shisa_principal *ph)
+			   const char *principal, Shisa_principal * ph)
 {
   Shisa_file *info = state;
 
   if (!_shisa_isdir3 (info->path, realm, principal))
     return SHISA_NO_PRINCIPAL;
 
+  if (!ph)
+    return SHISA_OK;
+
   ph->notusedbefore =
     _shisa_mtime4 (info->path, realm, principal, "validfrom.stamp");
   ph->isdisabled =
@@ -195,18 +217,6 @@ shisa_file_principal_find (Shisa * dbh,
   return SHISA_OK;
 }
 
-int
-shisa_file_principal_update (Shisa * dbh,
-			     void *state,
-			     const char *realm,
-			     const char *principal,
-			     const Shisa_principal * ph)
-{
-  Shisa_file *info = state;
-
-  return SHISA_OK;
-}
-
 static int
 realm_add (Shisa * dbh, void *state, const char *realm)
 {
@@ -227,8 +237,7 @@ principal_add (Shisa * dbh,
 	       void *state,
 	       const char *realm,
 	       const char *principal,
-	       const Shisa_principal * ph,
-	       const Shisa_key * key)
+	       const Shisa_principal * ph, const Shisa_key * key)
 {
   Shisa_file *info = state;
 
@@ -245,18 +254,19 @@ principal_add (Shisa * dbh,
     shisa_file_principal_update (dbh, state, realm, principal, ph);
 
   if (key)
-    shisa_file_key_add (dbh, state, realm, principal, ph->kvno, key);
+    shisa_file_key_add (dbh, state, realm, principal, key);
 
   return SHISA_OK;
 }
 
+/* Add new PRINCIPAL@REALM with specified information and key.  If
+   PRINCIPAL is NULL, then add realm REALM. */
 int
 shisa_file_principal_add (Shisa * dbh,
 			  void *state,
 			  const char *realm,
 			  const char *principal,
-			  const Shisa_principal * ph,
-			  const Shisa_key * key)
+			  const Shisa_principal * ph, const Shisa_key * key)
 {
   int rc;
 
@@ -268,6 +278,19 @@ shisa_file_principal_add (Shisa * dbh,
   return rc;
 }
 
+/* Modify information for specified PRINCIPAL@REALM.  */
+int
+shisa_file_principal_update (Shisa * dbh,
+			     void *state,
+			     const char *realm,
+			     const char *principal,
+			     const Shisa_principal * ph)
+{
+  Shisa_file *info = state;
+
+  return SHISA_OK;
+}
+
 static int
 realm_remove (Shisa * dbh, void *state, const char *realm)
 {
@@ -278,7 +301,8 @@ realm_remove (Shisa * dbh, void *state, const char *realm)
   if (!_shisa_isdir2 (info->path, realm))
     return SHISA_NO_REALM;
 
-  rc = shisa_file_enumerate_principals (dbh, state, realm, NULL, &nprincipals);
+  rc =
+    shisa_file_enumerate_principals (dbh, state, realm, NULL, &nprincipals);
   if (rc != SHISA_OK)
     return rc;
 
@@ -293,9 +317,7 @@ realm_remove (Shisa * dbh, void *state, const char *realm)
 
 static int
 remove_keys (Shisa * dbh,
-	     void *state,
-	     const char *realm,
-	     const char *principal)
+	     void *state, const char *realm, const char *principal)
 {
   Shisa_file *info = state;
   char **files;
@@ -326,9 +348,7 @@ remove_keys (Shisa * dbh,
 
 static int
 remove_info (Shisa * dbh,
-	     void *state,
-	     const char *realm,
-	     const char *principal)
+	     void *state, const char *realm, const char *principal)
 {
   Shisa_file *info = state;
   char **files;
@@ -355,9 +375,7 @@ remove_info (Shisa * dbh,
 
 static int
 principal_remove (Shisa * dbh,
-		  void *state,
-		  const char *realm,
-		  const char *principal)
+		  void *state, const char *realm, const char *principal)
 {
   Shisa_file *info = state;
   int rc;
@@ -378,11 +396,12 @@ principal_remove (Shisa * dbh,
   return SHISA_OK;
 }
 
+/* Remove PRINCIPAL@REALM, or REALM if PRINCIPAL is NULL.  Realms must
+   be empty for them to be successfully removed.  */
 int
 shisa_file_principal_remove (Shisa * dbh,
 			     void *state,
-			     const char *realm,
-			     const char *principal)
+			     const char *realm, const char *principal)
 {
   int rc;
 
@@ -396,11 +415,9 @@ shisa_file_principal_remove (Shisa * dbh,
 
 int
 read_key (Shisa * dbh,
-	  Shisa_file *info,
+	  Shisa_file * info,
 	  const char *realm,
-	  const char *principal,
-	  const char *keyfile,
-	  Shisa_key **key)
+	  const char *principal, const char *keyfile, Shisa_key ** key)
 {
   Shisa_key tmpkey;
   FILE *fh;
@@ -416,10 +433,15 @@ read_key (Shisa * dbh,
 
   memset (&tmpkey, 0, sizeof (tmpkey));
 
-  if (fscanf (fh, "%u %u %u %u %u\n", &tmpkey.etype, &tmpkey.keylen,
-	      &tmpkey.saltlen, &tmpkey.str2keyparamlen, &passwdlen) != 5)
+  rc = fscanf (fh, "%u %u %u %u %u %u\n", &tmpkey.etype, &tmpkey.keylen,
+	       &tmpkey.saltlen, &tmpkey.str2keyparamlen, &passwdlen,
+	       &tmpkey.priority);
+  if (rc != 5 && rc != 6)
     return SHISA_NO_KEY;
 
+  if (rc == 5)
+    tmpkey.priority = 0;
+
   if (tmpkey.keylen > 0)
     {
       tmpkey.key = xmalloc (tmpkey.keylen + 1);
@@ -456,7 +478,7 @@ read_key (Shisa * dbh,
   rc = fclose (fh);
   if (rc != 0)
     {
-      perror(keyfile);
+      perror (keyfile);
       return SHISA_NO_KEY;
     }
 
@@ -466,17 +488,46 @@ read_key (Shisa * dbh,
   return SHISA_OK;
 }
 
+static int
+key_match (const Shisa_key * hint, Shisa_key * key)
+{
+  int ok = 1;
+
+  if (hint->kvno)
+    ok = ok && hint->kvno == key->kvno;
+  if (hint->etype)
+    ok = ok && hint->etype == key->etype;
+  if (hint->keylen)
+    ok = ok && hint->keylen == key->keylen &&
+      memcmp (hint->key, key->key, key->keylen) == 0;
+  if (hint->saltlen)
+    ok = ok && hint->saltlen == key->saltlen &&
+      memcmp (hint->salt, key->salt, key->saltlen) == 0;
+  if (hint->str2keyparamlen)
+    ok = ok && hint->str2keyparamlen == key->str2keyparamlen &&
+      memcmp (hint->str2keyparam, key->str2keyparam,
+	      key->str2keyparamlen) == 0;
+  if (hint->password)
+    ok = ok && strcmp (hint->password, key->password) == 0;
+
+  return ok;
+}
+
+/* Get all keys matching HINT for specified PRINCIPAL@REALM.  The
+   caller must deallocate the returned keys.  If HINT is NULL, then
+   all keys are returned. */
 int
-shisa_file_enumerate_keys (Shisa * dbh,
-			   void *state,
-			   const char *realm,
-			   const char *principal,
-			   Shisa_key ***keys,
-			   size_t *nkeys)
+shisa_file_keys_find (Shisa * dbh,
+		      void *state,
+		      const char *realm,
+		      const char *principal,
+		      const Shisa_key * hint,
+		      Shisa_key *** keys, size_t * nkeys)
 {
   Shisa_file *info = state;
+  Shisa_key *tmpkey;
   char **files;
-  size_t nfiles;
+  size_t nfiles, matched = 0;
   size_t i;
   int rc;
 
@@ -485,38 +536,50 @@ shisa_file_enumerate_keys (Shisa * dbh,
 
   rc = _shisa_ls4 (info->path, realm, principal, "keys", &files, &nfiles);
   if (rc != SHISA_OK)
-    return rc;
+    return SHISA_ENUMERATE_KEY_ERROR;
 
-  *nkeys = nfiles;
+  if (nkeys)
+    *nkeys = nfiles;
   if (keys)
     *keys = xmalloc (nfiles * sizeof (**keys));
   for (i = 0; i < nfiles; i++)
     {
-      if (keys && rc == SHISA_OK)
+      if (rc == SHISA_OK &&
+	  (rc = read_key (dbh, info, realm, principal,
+			  files[i], &tmpkey)) == SHISA_OK)
 	{
-	  rc = read_key (dbh, info, realm, principal, files[i], &(*keys)[i]);
-	  if (rc != SHISA_OK)
-	    /* XXX mem leak. */
-	    (*keys)[i] = NULL;
+	  if (hint == NULL || key_match (hint, tmpkey))
+	    {
+	      if (keys)
+		(*keys)[matched] = tmpkey;
+	      matched++;
+	    }
+	  else
+	    shisa_key_free (dbh, tmpkey);
 	}
       free (files[i]);
     }
-  free (files);
 
-  return SHISA_OK;
+  if (nfiles > 0)
+    free (files);
+
+  if (nkeys)
+    *nkeys = matched;
+
+  return rc;
 }
 
+/* Add key for PRINCIPAL@REALM. */
 int
 shisa_file_key_add (Shisa * dbh,
 		    void *state,
 		    const char *realm,
-		    const char *principal,
-		    uint32_t kvno,
-		    const Shisa_key * key)
+		    const char *principal, const Shisa_key * key)
 {
   Shisa_file *info = state;
-  size_t passwdlen = key && key->password ? strlen(key->password) : 0;
-  char *file;
+  size_t passwdlen = key && key->password ? strlen (key->password) : 0;
+  char *file = NULL;
+  size_t num = 0;
   FILE *fh;
 
   if (!key)
@@ -526,17 +589,23 @@ shisa_file_key_add (Shisa * dbh,
       _shisa_mkdir4 (info->path, realm, principal, "keys"))
     return SHISA_NO_KEY;
 
-  asprintf (&file, "keys/%d.key", kvno);
+  do
+    {
+      if (file)
+	free (file);
+      asprintf (&file, "keys/%d-%d-%d.key", key->kvno, key->etype, num++);
+    }
+  while (_shisa_isfile4 (info->path, realm, principal, file));
   fh = _shisa_fopen4 (info->path, realm, principal, file, "w");
   free (file);
   if (!fh)
     {
       perror (file);
-      return SHISA_NO_KEY;
+      return SHISA_ADD_KEY_ERROR;
     }
 
-  fprintf (fh, "%u %u %u %u %u\n", key->etype, key->keylen,
-	   key->saltlen, key->str2keyparamlen, passwdlen);
+  fprintf (fh, "%u %u %u %u %u %u\n", key->etype, key->keylen,
+	   key->saltlen, key->str2keyparamlen, passwdlen, key->priority);
   if (key->keylen > 0)
     fwrite (key->key, 1, key->keylen, fh);
   if (key->saltlen > 0)
@@ -551,15 +620,78 @@ shisa_file_key_add (Shisa * dbh,
   return SHISA_OK;
 }
 
-void
-shisa_file_done (Shisa *dbh, void *state)
+/* Update a key for PRINCIPAL@REALM.  The OLDKEY must uniquely
+   determine the key to update, i.e., shishi_keys_find using OLDKEY as
+   HINT must return exactly 1 key.  */
+int
+shisa_file_key_update (Shisa * dbh,
+		       void *state,
+		       const char *realm,
+		       const char *principal,
+		       const Shisa_key * oldkey, const Shisa_key * newkey)
 {
   Shisa_file *info = state;
 
-  if (info)
+  puts ("fku");
+
+  return SHISA_NO_KEY;
+}
+
+/* Remove a key for PRINCIPAL@REALM.  The KEY must uniquely determine
+   the key to remove, i.e., shishi_keys_find using KEY as HINT must
+   return exactly 1 key.  */
+int
+shisa_file_key_remove (Shisa * dbh,
+		       void *state,
+		       const char *realm,
+		       const char *principal, const Shisa_key * key)
+{
+  Shisa_file *info = state;
+  Shisa_key *tmpkey;
+  char **files;
+  size_t nfiles;
+  size_t i;
+  int rc;
+  char *found = NULL;
+
+  files = NULL;
+  nfiles = 0;
+
+  rc = _shisa_ls4 (info->path, realm, principal, "keys", &files, &nfiles);
+  if (rc != SHISA_OK)
+    return rc;
+
+  for (i = 0; i < nfiles; i++)
     {
-      if (info->path)
-	free (info->path);
-      free (info);
+      if (rc == SHISA_OK &&
+	  (rc = read_key (dbh, info, realm, principal,
+			  files[i], &tmpkey)) == SHISA_OK)
+	{
+	  if (key == NULL || key_match (key, tmpkey))
+	    {
+	      if (found)
+		{
+		  free (found);
+		  rc = SHISA_MULTIPLE_KEY_MATCH;
+		}
+	      else
+		found = xstrdup (files[i]);
+	    }
+	  shisa_key_free (dbh, tmpkey);
+	}
+      free (files[i]);
     }
+
+  if (nfiles > 0)
+    free (files);
+
+  if (rc != SHISA_OK)
+    return rc;
+
+  if (!found)
+    return SHISA_NO_KEY;
+
+  rc = _shisa_rm5 (info->path, realm, principal, "keys", found);
+  free (found);
+  return rc;
 }
