@@ -1,4 +1,4 @@
-/* crypto-3des.c	3DES crypto functions
+/* crypto-3des.c --- 3DES crypto functions.
  * Copyright (C) 2002, 2003  Simon Josefsson
  *
  * This file is part of Shishi.
@@ -21,8 +21,12 @@
 
 #include "internal.h"
 
+/* Get prototypes. */
 #include "crypto.h"
 
+/* Get _shishi_escapeprint, etc. */
+#include "utils.h"
+
 static int
 _des3_encrypt (Shishi * handle,
 	       Shishi_key * key,
