@@ -1,5 +1,5 @@
-/* shishi.h	Header file for Shishi library.                       -*- c -*-
- * Copyright (C) 2002, 2003  Simon Josefsson
+/* shishi.h --- Header file for Shishi library.                       -*- c -*-
+ * Copyright (C) 2002, 2003, 2004  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.11"
+#define SHISHI_VERSION "0.0.12"
 
 /* Error codes */
 typedef enum
@@ -285,6 +285,7 @@ typedef enum
   SHISHI_HMAC_SHA1_96_AES128 = 15,
   SHISHI_HMAC_SHA1_96_AES256 = 16,
   SHISHI_ARCFOUR_HMAC_MD5 = -138,
+  SHISHI_KRB5_GSSAPI_CKSUM = 8003,
   SHISHI_NO_CKSUMTYPE = -1
 }
 Shishi_cksumtype;
@@ -655,6 +656,7 @@ extern char *shishi_server_for_local_service (Shishi * handle,
 					      const char *service);
 
 /* ticket.c */
+extern Shishi_asn1 shishi_ticket (Shishi * handle);
 extern int shishi_ticket_server (Shishi * handle, Shishi_asn1 ticket,
 				 char **server, size_t * serverlen);
 extern int shishi_ticket_sname_get (Shishi * handle,
@@ -695,6 +697,7 @@ extern int shishi_ticket_decrypt (Shishi * handle, Shishi_asn1 ticket,
 
 /* tkt.c */
 extern Shishi_asn1 shishi_tkt_ticket (Shishi_tkt * tkt);
+extern void shishi_tkt_ticket_set (Shishi_tkt * tkt, Shishi_asn1 ticket);
 extern Shishi_asn1 shishi_tkt_kdcrep (Shishi_tkt * tkt);
 extern Shishi_asn1 shishi_tkt_enckdcreppart (Shishi_tkt * tkt);
 extern void shishi_tkt_enckdcreppart_set (Shishi_tkt * tkt,
@@ -743,7 +746,7 @@ extern int shishi_tkt_keytype (Shishi_tkt * tkt, int32_t * etype);
 extern int32_t shishi_tkt_keytype_fast (Shishi_tkt * tkt);
 extern int shishi_tkt_keytype_p (Shishi_tkt * tkt, int32_t etype);
 extern int shishi_tkt_server (Shishi_tkt * tkt,
-			      char *service, size_t * servicelen);
+			      char **server, size_t * serverlen);
 extern int shishi_tkt_server_p (Shishi_tkt * tkt, const char *server);
 extern int shishi_tkt_server_realm (Shishi_tkt * tkt,
 				    char *serverrealm,
@@ -755,6 +758,7 @@ extern int shishi_tkt_decrypt (Shishi_tkt * tkt, Shishi_key * key);
 extern void shishi_tkt_done (Shishi_tkt * tkt);
 extern int shishi_tkt_flags (Shishi_tkt * tkt, int *flags);
 extern int shishi_tkt_flags_set (Shishi_tkt * tkt, int flags);
+extern int shishi_tkt_flags_add (Shishi_tkt * tkt, int flag);
 extern int shishi_tkt_forwardable_p (Shishi_tkt * tkt);
 extern int shishi_tkt_forwarded_p (Shishi_tkt * tkt);
 extern int shishi_tkt_proxiable_p (Shishi_tkt * tkt);
@@ -1042,7 +1046,7 @@ extern int shishi_asreq_cnamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
 extern int shishi_kdcreq_nonce (Shishi * handle, Shishi_asn1 kdcreq,
 				uint32_t * nonce);
 extern int shishi_kdcreq_client (Shishi * handle, Shishi_asn1 kdcreq,
-				 char **client, size_t *clientlen);
+				 char **client, size_t * clientlen);
 extern int shishi_kdcreq_cname_get (Shishi * handle,
 				    Shishi_asn1 kdcreq,
 				    char *cname, size_t * cnamelen);
@@ -1050,7 +1054,7 @@ extern int shishi_kdcreq_set_cname (Shishi * handle, Shishi_asn1 kdcreq,
 				    Shishi_name_type name_type,
 				    const char *principal);
 extern int shishi_kdcreq_server (Shishi * handle, Shishi_asn1 kdcreq,
-				 char **server, size_t *serverlen);
+				 char **server, size_t * serverlen);
 extern int shishi_kdcreq_sname_get (Shishi * handle, Shishi_asn1 kdcreq,
 				    char *sname, size_t * snamelen);
 extern int shishi_kdcreq_snamerealm_get (Shishi * handle, Shishi_asn1 kdcreq,
@@ -1075,7 +1079,21 @@ extern int shishi_kdcreq_set_etype (Shishi * handle, Shishi_asn1 kdcreq,
 				    int32_t * etype, int netype);
 extern int shishi_kdcreq_options (Shishi * handle, Shishi_asn1 kdcreq,
 				  uint32_t * flags);
+extern int shishi_kdcreq_forwardable_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_forwarded_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_proxiable_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_proxy_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_allow_postdate_p (Shishi * handle,
+					   Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_postdated_p (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_renewable_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_disable_transited_check_p (Shishi * handle,
+						    Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_renewable_ok_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_enc_tkt_in_skey_p (Shishi * handle,
+					    Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_renew_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_validate_p (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_options_set (Shishi * handle, Shishi_asn1 kdcreq,
 				      uint32_t options);
 extern int shishi_kdcreq_options_add (Shishi * handle, Shishi_asn1 kdcreq,
@@ -1664,6 +1682,7 @@ extern int shishi_ap_req_der_set (Shishi_ap * ap, char *der, size_t derlen);
 extern int shishi_ap_req_build (Shishi_ap * ap);
 extern int shishi_ap_req_asn1 (Shishi_ap * ap, Shishi_asn1 * apreq);
 extern Shishi_key *shishi_ap_key (Shishi_ap * ap);
+extern int shishi_ap_req_decode (Shishi_ap * ap);
 extern int shishi_ap_req_process (Shishi_ap * ap, Shishi_key * key);
 extern int shishi_ap_req_process_keyusage (Shishi_ap * ap,
 					   Shishi_key * key,
