@@ -50,7 +50,7 @@ shishi_tkt (Shishi * handle, Shishi_tkt ** tkt)
 
   t->handle = handle;
 
-  t->ticket = shishi_asn1_ticket (handle);
+  t->ticket = shishi_ticket (handle);
   if (t->ticket == NULL)
     {
       shishi_error_printf (handle, "Could not create Ticket: %s\n",
@@ -177,6 +177,21 @@ shishi_tkt_ticket (Shishi_tkt * tkt)
   return tkt->ticket;
 }
 
+/**
+ * shishi_tkt_ticket_set:
+ * @tkt: input variable with ticket info.
+ * @ticket: ASN.1 Ticket to store in ticket.
+ *
+ * Set the ASN.1 Ticket in the Ticket.
+ **/
+void
+shishi_tkt_ticket_set (Shishi_tkt * tkt, Shishi_asn1 ticket)
+{
+  if (tkt->ticket)
+    shishi_asn1_done (tkt->handle, tkt->ticket);
+  tkt->ticket = ticket;
+}
+
 /**
  * shishi_tkt_enckdcreppart:
  * @tkt: input variable with ticket info.
@@ -191,7 +206,7 @@ shishi_tkt_enckdcreppart (Shishi_tkt * tkt)
 
 /**
  * shishi_tkt_encticketreppart_set:
- * @as: structure that holds information about Ticket exchange
+ * @tkt: structure that holds information about Ticket exchange
  * @enckdcreppart: EncKDCRepPart to store in Ticket.
  *
  * Set the EncKDCRepPart in the Ticket.
@@ -456,11 +471,25 @@ shishi_tkt_realm (Shishi_tkt * tkt, char **realm, size_t * realmlen)
   return shishi_ticket_realm_get (tkt->handle, tkt->ticket, realm, realmlen);
 }
 
+/**
+ * shishi_tkt_server:
+ * @tkt: input variable with ticket info.
+ * @server: pointer to newly allocated zero terminated string containing
+ *   principal name.  May be %NULL (to only populate @serverlen).
+ * @serverlen: pointer to length of @server on output, excluding terminating
+ *   zero.  May be %NULL (to only populate @server).
+ *
+ * Represent server principal name in Ticket as zero-terminated
+ * string.  The string is allocate by this function, and it is the
+ * responsibility of the caller to deallocate it.  Note that the
+ * output length @serverlen does not include the terminating zero.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
 int
-shishi_tkt_server (Shishi_tkt * tkt, char *server, size_t * serverlen)
+shishi_tkt_server (Shishi_tkt * tkt, char **server, size_t * serverlen)
 {
-  return shishi_ticket_sname_get (tkt->handle, tkt->ticket,
-				  server, serverlen);
+  return shishi_ticket_server (tkt->handle, tkt->ticket, server, serverlen);
 }
 
 /**
@@ -476,29 +505,17 @@ int
 shishi_tkt_server_p (Shishi_tkt * tkt, const char *server)
 {
   char *buf;
-  size_t buflen;
   int res;
 
-  buflen = strlen (server) + 1;
-  buf = xmalloc (buflen);
-
-  res = shishi_tkt_server (tkt, buf, &buflen);
+  res = shishi_tkt_server (tkt, &buf, NULL);
   if (res != SHISHI_OK)
-    {
-      free (buf);
-      return 0;
-    }
-  buf[buflen] = '\0';
+    return 0;
 
-  if (strcmp (server, buf) != 0)
-    {
-      free (buf);
-      return 0;
-    }
+  res = strcmp (server, buf) == 0;
 
   free (buf);
 
-  return 1;
+  return res;
 }
 
 int
@@ -514,7 +531,7 @@ shishi_tkt_server_realm (Shishi_tkt * tkt,
  * @tkt: input variable with ticket info.
  * @flags: pointer to output integer with flags.
  *
- * Extract flags in ticket.
+ * Extract flags in ticket (i.e., EncKDCRepPart).
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
@@ -530,8 +547,8 @@ shishi_tkt_flags (Shishi_tkt * tkt, int *flags)
  * @tkt: input variable with ticket info.
  * @flags: integer with flags to store in ticket.
  *
- * Set flags in ticket.  Note that this reset any already existing
- * flags.
+ * Set flags in ticket, i.e., both EncTicketPart and EncKDCRepPart.
+ * Note that this reset any already existing flags.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
@@ -553,6 +570,35 @@ shishi_tkt_flags_set (Shishi_tkt * tkt, int flags)
   return SHISHI_OK;
 }
 
+/**
+ * shishi_tkt_flags_add:
+ * @tkt: input variable with ticket info.
+ * @flag: integer with flags to store in ticket.
+ *
+ * Add ticket flags to Ticket and EncKDCRepPart.  This preserves all
+ * existing options.
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
+int
+shishi_tkt_flags_add (Shishi_tkt * tkt, int flag)
+{
+  int flags;
+  int res;
+
+  res = shishi_tkt_flags (tkt, &flags);
+  if (res != SHISHI_OK)
+    return res;
+
+  flags |= flag;
+
+  res = shishi_tkt_flags_set (tkt, flags);
+  if (res != SHISHI_OK)
+    return res;
+
+  return SHISHI_OK;
+}
+
 /**
  * shishi_tkt_forwardable_p:
  * @tkt: input variable with ticket info.
@@ -1308,6 +1354,7 @@ void
 shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh)
 {
   char buf[BUFSIZ];
+  char *buf2;
   char *p;
   size_t buflen;
   int keytype, etype, flags;
@@ -1350,12 +1397,9 @@ shishi_tkt_pretty_print (Shishi_tkt * tkt, FILE * fh)
   if (t != (time_t) - 1)
     fprintf (fh, _("Renewable till:\t%s"), ctime (&t));
 
-  buflen = sizeof (buf);
-  buf[0] = '\0';
-  res = shishi_tkt_server (tkt, buf, &buflen);
-  buf[buflen] = '\0';
+  res = shishi_tkt_server (tkt, &buf2, NULL);
   res = shishi_ticket_get_enc_part_etype (tkt->handle, tkt->ticket, &keytype);
-  fprintf (fh, _("Server:\t\t%s key %s (%d)\n"), buf,
+  fprintf (fh, _("Server:\t\t%s key %s (%d)\n"), buf2,
 	   shishi_cipher_name (keytype), keytype);
 
   res = shishi_tkt_keytype (tkt, &keytype);
